//*****************************************
// AT mega328 Pulse Output Program
// CPU Clock:20MHz
//   iq[Yf[^@WȂBOIV[^(8MHz--)ɂj
//
// File Name:timer.c
//
//  2016/02/26 Taishi
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>

#include	"pulse.h"

#include	"timer.h"

// **********************************************************
// ********************** Soft Timer ************************
// **********************************************************

volatile unsigned int	wait_i;
volatile unsigned int	wait_j;
volatile unsigned char	wait_us;
#if	CPU_CLOCK == 20
// ************************************
// Delay 2uS
// t: uS (max 255)
// CPU Clock:20MHz
//  while ߂ for ŎԂ킹Ă
// ************************************
void delay_2us(unsigned char t)
{
	unsigned char	i;

	for(i=0;i<t;i++){
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
	}
}
#else
// ************************************
// Delay 2uS
// t: uS (max 255)
// CPU Clock:8.000MHz
//  while ߂ for ŎԂ킹Ă
// ************************************
void delay_2us(unsigned char t)
{
	unsigned char	i;

	for(i=0;i<t;i++){
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
	}
}
#endif

//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
// CPU Clock:1MHz
//***************************
void	delay_ms(int tm)
{

	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_2us(500/2);
		delay_2us(500/2);
	}
}

// **********************************************************
// ************************ Timer 1 *************************
// **********************************************************

int		pulse_out;
int		pulse_period;
int		pulse_unit;
static	unsigned char	tim1_unit[5]={
	(1 << CS10),					// 1/1
	(1 << CS11),					// 1/8
	((1 << CS11) | (1 << CS10)),	// 1/64
	(1 << CS12),					// 1/256
	((1 << CS12) | (1 << CS10)),	// 1/1024
};
//*****************************************
// Timer1 close
//*****************************************
void	timer1_close(void)
{
	TCCR1B = 0x00;														// Timer1 Stop

	DDRB |= (1 << DDB1);												// Port-B Bit1 : Output
	PORTB &= ~(1 << PB1);												// Port-B Bit1 = Low
	TCNT1 = 0;															// Timer1 Count Set to ZERO
	TCCR1A = 0;															//
	TCCR1B = 0;															//

}

//*****************************************
// Pulse ON
// \ timer1  OCR1A,ICR1̃WX^̓ZbgĂKv
//*****************************************
void	pulse_on(void)
{
	unsigned char	ud;

	TCCR1A = ( (1 << COM1A1) | (0 << COM1A0) | (1 << WGM11) | (0 << WGM10) );	// PWM TOP=ICR1, OC1A:BOTTOM=H Match=L, 
	ud = TCCR1B;										// read Timer1 Prescaler
	ud &= ~( (1 << CS12) | (1 << CS11) | (1 << CS10) );	// Timer1 Prescaler Clear
	ud |= tim1_unit[pulse_unit];						// Timer1 Prescaler data Set
	TCCR1B = ud;										// Timer1 Prescaler Set
}
//*****************************************
// Pulse OFF
//*****************************************
void	pulse_off(void)
{
	TCCR1A = ( (0 << COM1A1) | (0 << COM1A0) | (1 << WGM11) | (0 << WGM10) );	// Standerd Port TOP=ICR1, OC1A:BOTTOM=H Match=L, 
	TCCR1B &= ~( (1 << CS12) | (1 << CS11) | (1 << CS10) );						// Timer1 Prescaler Stop
	TCNT1 = 0;																	// Timer1 Count Set to ZERO
}
//*****************************************
// Set Timer1 ON Time
//*****************************************
void	set_timer1_ontime(void)
{
	unsigned int	ud;

	if(PULSE_OUT_MAX <= pulse_out){
		ud = PULSE_OUT_MAX;
	}else if(pulse_out <= PULSE_OUT_MIN){
		ud = PULSE_OUT_MIN;
	}else{
		ud = (unsigned int)pulse_out;
	}

	ud--;										// xRJEgA߂ɃY݂Ȃ̂Ł@PJEg炷

	OCR1A  = ud;								// ON Time ( 1=125nS : 8MHz)
}
//*****************************************
// Set Timer1 Period
//*****************************************
void	set_timer1_period(void)
{
	unsigned int	ud;

	ud = (unsigned int)pulse_period;

	ud--;									// xRJEgA߂ɃY݂Ȃ̂Ł@PJEg炷

	ICR1  = ud;								// Period Time ( 1=125nS : 8MHz)
}
//*****************************************
// Timer1 Initial
//  PD4(OC1B): Pulse Out for High Volt Generator
//*****************************************
void	timer1_init(void)
{
	TCCR1B = 0x00;														// Timer1 Stop
	PRR &= ~( 1 << PRTIM1 );											// Timer1 ғ

	PORTB &= ~(1 << PB1);												// Port-B Bit1=Low
	DDRB |= (1 << DDB1);												// Port-B Bit1 : Output

	pulse_out = PULSE_OUT_DEFAULT;										// pXnm
	pulse_period = PULSE_PERIOD_DEFAULT;								// pX
	pulse_unit = PULSE_UNIT_DEFAULT;									// PpXP

	TCNT1 = 0;															// Timer1 Count Set to ZERO
	TCCR1A = ( (1 << COM1A1) | (0 << COM1A0) | (1 << WGM11) | (0 << WGM10) );	// PWM TOP=ICR1, OC1A:BOTTOM=H Match=L, 
	TCCR1B = ( (1 << WGM13) | (1 << WGM12) );							// A Comper Match

	set_timer1_period();												// 8MHz / 1000 = 8kHz(125uS)
	OCR1A  = (unsigned int)pulse_out;									// On Time ( 1=125nS)

}

//***************************
// Timer1 Comper Match A Interrupt
//
//  ̏125uS Ԋu
// Ă΂Ȃ͂
//***************************
ISR	(TIMER1_COMPA_vect)	
{
}
//***************************
// Timer1 Comper Match B Interrupt
//
//  ̏125uS Ԋu
// Ă΂Ȃ͂
//***************************
ISR	(TIMER1_COMPB_vect)	
{
}
// **********************************************************
// ************************ Timer 2 *************************
// **********************************************************
#if	CPU_CLOCK == 20
#else
unsigned char	tim_1ms;						// 10mSPʂJE^[
#endif
unsigned char	tim_10ms;						// PbPʂJE^[
unsigned char	tim_10ms_sw;					// XCb`͗pPOrPʃ^C}[
unsigned char	tim_10ms_blink;					// \uNpPOrPʃ^C}[
//*****************************************
// Timer2 Initial
//  1mS 荞݂ɐݒ肷
//*****************************************
void	timer2_init(void)
{
	TCCR2B = 0;											// Timer Stop
	PRR &= ~( 1 << PRTIM2 );							// Timer2 ғ
	TCNT2 = 0;											// Timer2 Count Set to ZERO
	TCCR2A = ((1 << WGM21) | (0 << WGM20));				// A Comper Match, Dont Output Timer2 Signal
#if	CPU_CLOCK == 20
	OCR2A  = COUNT_10MS;								// about 10mS at 20MHz/1024 = 19531.25Hz
	TCCR2B = D_TCCR2B_10MS;								// Timer Start ((20MHz/1024= 19531.25Hz 51.2uS))
#else
	OCR2A  = COUNT_1MS;									// 1mS 8MHz/64=125kHz
	TCCR2B = D_TCCR2B_1MS;								// Timer Start (8MHz/64= 125kHz, 8uS)
#endif
	TIMSK2 |= (1 << OCIE2A);							// A Comper Match Interrupt Enable

#if	CPU_CLOCK == 20
#else
	tim_1ms = 1;										// 10mSPʂJE^[
#endif
	tim_10ms = 0;										// PbPʂJE^[
	tim_10ms_sw = 0;									// XCb`͗pPOrPʃ^C}[
	tim_10ms_blink = 0;									// \uNpPOrPʃ^C}[

}
//*****************************************
// Timer2 Close
//  Timer2 ̎gp߂
//*****************************************
void	timer2_close(void)
{
	TCCR2B = 0;											// Timer Stop
	TCNT2 = 0;											// Timer2 Count Set to ZERO
	TCCR2A = 0;											// Dont Output Timer2 Signal
	TIMSK2 = 0;											// Interrupt Disable

}
#if	CPU_CLOCK == 20
//***************************
// Timer2 Comper Match A Interrupt
//
//  10mS 
//***************************
ISR	(TIMER2_COMPA_vect)	
{
	tim_10ms++;
	if(100 <= tim_10ms){					// Pbo
		tim_10ms = 0;
	}

	if(tim_10ms_sw != 0){
		tim_10ms_sw--;
	}

	if(tim_10ms_blink != 0){
		tim_10ms_blink--;
	}
}
#else
//***************************
// Timer2 Comper Match A Interrupt
//
//  1mS 
//***************************
ISR	(TIMER2_COMPA_vect)	
{
	tim_1ms--;
	if(tim_1ms == 0){
		tim_1ms = 10;
		tim_10ms++;
		if(100 <= tim_10ms){					// Pbo
			tim_10ms = 0;
		}

		if(tim_10ms_sw != 0){
			tim_10ms_sw--;
		}

		if(tim_10ms_blink != 0){
			tim_10ms_blink--;
		}

	}
}
#endif
